//  Copyright (c) 2006, Wade Tregaskis.  All rights reserved.
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//    * Neither the name of Wade Tregaskis nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#ifndef PERFORMANCEMETRICS_H
#define PERFORMANCEMETRICS_H

#include <vector>
using namespace std;


// Yes, it's not pretty having global variables like this, but we're
// interested in performance here, and this offers the best of all
// possible ways of doing this.
long long unsigned int comparisonCount = 0, swapCount = 0, insertCount = 0, removeCount = 0;


// The reason I use the following functions, rather than the STL equivalents, is because
// this is a very easy way of keeping track of how many swaps, inserts and removes are
// performed.  And when inlined the only real difference is the convenience of having the
// code only in this one place.

template<class V> void swap(V *(&a), V *(&b)) {
    register V *temp = a;

    a = b;
    b = temp;

    ++swapCount;
}

template<class V, class A> void insert(vector<V*, A> &vec, typename vector<V*, A>::iterator iter, V *item) {
    vec.insert(iter, item);
    ++insertCount;
}

template<class V, class A> void remove(vector<V*, A> &vec, typename vector<V*, A>::iterator iter) {
    ++removeCount;
    vec.erase(iter);
}

#endif
