//  Copyright (c) 2006, Wade Tregaskis.  All rights reserved.
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//    * Neither the name of Wade Tregaskis nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#ifndef VOTER_H
#define VOTER_H

#include <string>
#include "PerformanceMetrics.h"
using namespace std;


class Voter {
public:
    Voter(const Voter &original) : _familyName(original._familyName), _givenNames(original._givenNames), _address(original._address), _suburb(original._suburb), _state(original._state), _postcode(original._postcode) {}

    Voter(const string &familyName = "None", const string &givenNames = "None", const string &address = "None", const string &suburb = "None", const string &state = "None", unsigned short postcode = 1234) : _familyName(familyName), _givenNames(givenNames), _address(address), _suburb(suburb), _state(state), _postcode(postcode) {}

    const string& familyName(void) const { return _familyName; }
    const string& givenNames(void) const { return _givenNames; }
    const string& address(void) const { return _address; }
    const string& suburb(void) const { return _suburb; }
    const string& state(void) const { return _state; }
    unsigned short postcode(void) const { return _postcode; }

    void setFamilyName(const string &familyName) { _setFamilyName(familyName); }
    void setGivenNames(const string &givenNames) { _setGivenNames(givenNames); }
    void setAddress(const string &address) { _setAddress(address); }
    void setSuburb(const string &suburb) { _setSuburb(suburb); }
    void setState(const string &state) { _setState(state); }
    void setPostcode(unsigned short postcode) { _setPostcode(postcode); }

    Voter& operator=(const Voter &original) {
        if (this != &original) {
            _setFamilyName(original.familyName());
            _setGivenNames(original.givenNames());
            _setAddress(original.address());
            _setSuburb(original.suburb());
            _setState(original.state());
            _setPostcode(original.postcode());
        }

        return *this;
    }

    friend istream& operator>>(istream& in, Voter &voter);
    friend ostream& operator<<(ostream& out, const Voter &voter);

protected:
        inline void _setFamilyName(const string &familyName) { _familyName = string(familyName); }
    inline void _setGivenNames(const string &givenNames) { _givenNames = string(givenNames); }
    inline void _setAddress(const string &address) { _address = string(address); }
    inline void _setSuburb(const string &suburb) { _suburb = string(suburb); }
    inline void _setState(const string &state) { _state = string(state); }
    inline void _setPostcode(unsigned short postcode) { _postcode = postcode; }

    string _familyName, _givenNames, _address, _suburb, _state;
    unsigned short _postcode;
};

bool operator==(const Voter &a, const Voter &b) {
    return ((a.familyName() == b.familyName()) && (a.givenNames() == b.givenNames()) && (a.address() == b.address()) && (a.suburb() == b.suburb()) && (a.state() == b.state()) && (a.postcode() == b.postcode()));
}

bool operator!=(const Voter &a, const Voter &b) {
    return !(a == b);
}

bool operator<(const Voter &a, const Voter &b) {
    register int stringCompareResult;

    ++comparisonCount;

    if (a.postcode() == b.postcode()) {
        stringCompareResult = strcasecmp(a.familyName().c_str(), b.familyName().c_str());

        if (stringCompareResult == 0) {
            stringCompareResult = strcasecmp(a.givenNames().c_str(), b.givenNames().c_str());

            if (stringCompareResult < 0) {
                return true;
            } else {
                return false;
            }
        } else if (stringCompareResult < 0) {
            return true;
        } else {
            return false;
        }
    } else if (a.postcode() < b.postcode()) {
        return true;
    } else { // a > b
        return false;
    }
}

bool operator>(const Voter &a, const Voter &b) {
    register int stringCompareResult;

    ++comparisonCount;

    if (a.postcode() == b.postcode()) {
        stringCompareResult = strcasecmp(a.familyName().c_str(), b.familyName().c_str());

        if (stringCompareResult == 0) {
            stringCompareResult = strcasecmp(a.givenNames().c_str(), b.givenNames().c_str());

            if (stringCompareResult > 0) {
                return true;
            } else {
                return false;
            }
        } else if (stringCompareResult > 0) {
            return true;
        } else {
            return false;
        }
    } else if (a.postcode() > b.postcode()) {
        return true;
    } else { // a < b
        return false;
    }
}

bool operator<=(const Voter &a, const Voter &b) {
    register int stringCompareResult;

    ++comparisonCount;

    if (a.postcode() == b.postcode()) {
        stringCompareResult = strcasecmp(a.familyName().c_str(), b.familyName().c_str());

        if (stringCompareResult == 0) {
            stringCompareResult = strcasecmp(a.givenNames().c_str(), b.givenNames().c_str());

            if (stringCompareResult <= 0) {
                return true;
            } else {
                return false;
            }
        } else if (stringCompareResult < 0) {
            return true;
        } else {
            return false;
        }
    } else if (a.postcode() < b.postcode()) {
        return true;
    } else { // a > b
        return false;
    }
}

bool operator>=(const Voter &a, const Voter &b) {
    return (b <= a);
}

istream& operator>>(istream& in, Voter &voter) {
    static string postcode;

    getline(in, voter._familyName, ',');
    in.ignore(1, ' ');
    getline(in, voter._givenNames);
    getline(in, voter._address);
    getline(in, voter._suburb, '\t');
    getline(in, voter._state, '\t');
    getline(in, postcode);

    in.ignore(10000, '\n');

    voter._postcode = atoi(postcode.c_str());

    return in;
}

ostream& operator<<(ostream& out, const Voter &voter) {
    out << voter._familyName << ", " << voter._givenNames << '\n' << voter._address << '\n' << voter._suburb << '\t' << voter._state << '\t' << voter._postcode << '\n';

    return out;
}

#endif
