//  Copyright (c) 2006, Wade Tregaskis.  All rights reserved.
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//    * Neither the name of Wade Tregaskis nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include "Timing.h"
#include "Voter.h"
#include "PerformanceMetrics.h"
#include "AVL.h"
#include "SortAlgorithms.h"
using namespace std;


// Big Beefy Brains

int main (int argc, const char * argv[]) {
    ifstream input;
    ofstream output;
    unsigned int recordCount;
    vector<Voter*> role;
    vector<Voter*>::const_iterator roleIterator;
    Voter *tempInput;
    double cpuStamp, timeStamp, cpuTaken, timeTaken;
    
    if (argc < 4) {
        cout << "Usage: " << argv[0] << " M INFILE OUTFILE\n\n\tM\tSort mode - Insertion, Selection, Bubble, Quicksort, Mergesort, Radix, AVL\n\tINFILE\tThe file from which to input data\n\tOUTFILE\tThe file to output the sorted data to\n";
    } else {
        input.open(argv[2]);

        if (input.fail()) {
            cout << "Unable to open \"" << argv[2] << "\" for reading.\n";
        } else {
            output.open(argv[3]);

            if (output.fail()) {
                cout << "Unable to open \"" << argv[3] << "\" for writing.\n";
            } else {
                while (1) {
                    tempInput = new Voter;
                    
                    input >> *tempInput;

                    if (!input.eof()) {
                        role.push_back(tempInput);
                    } else {
                        delete tempInput;
                        break;
                    }
                }

                recordCount = role.size();
                //cout << "Read in " << role.size() << " records from \"" << argv[2] << "\".\n"; // only needed for debugging

                timeStamp = timeSoFar();
                cpuStamp = cpuSoFar();
                
                switch (toupper(argv[1][0])) {
                    case 'I':
                        cout << "Performing insertion sort...";
                        performInsertionSort(role); break;
                    case 'S':
                        cout << "Performing selection sort...";
                        performSelectionSort(role); break;
                    case 'B':
                        cout << "Performing bubble sort...";
                        performBubbleSort(role); break;
                    case 'Q':
                        cout << "Performing quick sort...";
                        performQuickSort(role); break;
                    case 'M':
                        cout << "Performing merge sort...";
                        performMergeSort(role); break;
                    case 'R':
                        cout << "Performing optimized radix sort...";
                        performOptimizedRadixSort(role); break;
                    case 'A':
                        cout << "Performing AVL tree sort...";
                        performAVLTreeSort(role); break;
                    default:
                        cout << "Unknown sort type, \"" << argv[1][0] << "\".  Please check the type and try again.\n";
                }

                timeTaken = timeSoFar() - timeStamp;
                cpuTaken = cpuSoFar() - cpuStamp;

                if (recordCount != role.size()) {
                    // Detect any lost or gained records... don't laugh, radix sort did this at one point
                    cout << "ERROR - " << recordCount << " records were read in, but " << role.size() << " were written out";
                }
                
                cout << "\nTime: " << timeTaken << " - CPU: " << cpuTaken << " (" << ((cpuTaken / timeTaken) * 100.0) << "%)\nComparisons: " << comparisonCount << "\nSwaps: " << swapCount << "\nInserts: " << insertCount << "\nRemoves: " << removeCount << '\n';

                for (roleIterator = role.begin(); roleIterator != role.end(); ++roleIterator) {
                    output << *(*roleIterator) << '\n';
                }

                output.close();

                //cout << role.size() << " records written to \"" << argv[3] << "\".\n"; // only needed for debugging
            }

            input.close();
        }
    }

    return 0;
}
